import boto3
import datetime
import time
import json
from objict import objict
from rest import settings
from concurrent.futures import ThreadPoolExecutor


LOG_CACHE = objict()


def getClient():
    if LOG_CACHE.client is None:
        key = settings.AWS_KEY
        secret = settings.AWS_SECRET
        region = settings.AWS_REGION
        LOG_CACHE.client = boto3.client("logs", aws_access_key_id=key, aws_secret_access_key=secret, region_name=region)
    return LOG_CACHE.client


def log(data, log_group, log_stream):
    if LOG_CACHE.pool is None:
        LOG_CACHE.pool = ThreadPoolExecutor(max_workers=1)
    LOG_CACHE.pool.submit(logToCloudWatch, data, log_group, log_stream)
    return True


def logToCloudWatch(data, log_group, log_stream):
    message = data
    if isinstance(message, dict):
        message = json.dumps(message)
    return logBatchToCloudWatch([
            dict(
                timestamp=int(datetime.datetime.utcnow().timestamp() * 1000),
                message=message)
        ], log_group, log_stream)


def logBatchToCloudWatch(batch, log_group, log_stream):
    return getClient().put_log_events(
        logGroupName=log_group,
        logStreamName=log_stream,
        logEvents=batch
    )


def getLogGroups():
    response = getClient().describe_log_groups()
    return response.get('logGroups', [])


def createLogStream(log_group, log_stream):
    try:
        getClient().create_log_stream(logGroupName=log_group, logStreamName=log_stream)
    except Exception:
        pass  # Log stream already exists, no need to create it


def getInsights(log_group, start_time, end_time, query_string):
    """
    Executes a CloudWatch Logs Insights query and returns the results.

    :param log_group: The name of the log group to query.
    :param start_time: The start time of the query (epoch time in seconds).
    :param end_time: The end time of the query (epoch time in seconds).
    :param query_string: The query string to use.
    :param region_name: AWS region name.
    :return: The query results.
    """
    # Create a CloudWatch Logs client
    client = getClient()
    
    # Start the query
    start_query_response = client.start_query(
        logGroupName=log_group,
        startTime=start_time,
        endTime=end_time,
        queryString=query_string,
    )
    
    query_id = start_query_response['queryId']
    
    # Wait for the query to complete
    response = None
    while response is None or response['status'] == 'Running':
        time.sleep(1)  # Sleep to rate limit the polling
        response = client.get_query_results(queryId=query_id)
    
    if response['status'] == 'Complete':
        return response['results']
    else:
        raise Exception(f"Query did not complete successfully. Status: {response['status']}")



import json
import requests
import time
from jwt import JWT, jwk_from_dict
from datetime import datetime, timedelta


def oauthLogin(service_account_info):
    # Token endpoint
    token_uri = service_account_info["token_uri"]

    # The current time and expiration time for the assertion
    issued_at_time = datetime.utcnow()
    expiration_time = issued_at_time + timedelta(minutes=60)

    # JWT Header
    jwt_header = {
        "alg": "RS256",
        "typ": "JWT",
        "kid": service_account_info["private_key_id"]
    }

    # JWT Payload
    jwt_payload = {
        "iss": service_account_info["client_email"],
        "sub": service_account_info["client_email"],
        "aud": token_uri,
        "iat": int(issued_at_time.timestamp()),
        "exp": int(expiration_time.timestamp()),
        "scope": "https://www.googleapis.com/auth/firebase.messaging"
    }

    # Create a JWT
    jwt_instance = JWT()
    private_key = jwk_from_dict({"k": service_account_info["private_key"], "kty": "RSA"})
    assertion = jwt_instance.encode(jwt_header, jwt_payload, private_key)

    # Exchange the JWT for an access token
    response = requests.post(token_uri, data={
        "grant_type": "urn:ietf:params:oauth:grant-type:jwt-bearer",
        "assertion": assertion
    })

    response_data = response.json()

    access_token = response_data.get("access_token")
    expires_in = response_data.get("expires_in")  # Seconds until the token expires

    # You can now use this access_token to authenticate requests to Google APIs.
    # Remember to refresh the token using a similar process once it's close to expiration.

    print("Access Token:", access_token)
    print("Expires In:", expires_in)



from google.oauth2 import service_account
import google.auth.transport.requests
import requests
from rest import settings
from rest import log


FCM_ENDPOINT = 'https://fcm.googleapis.com/v1/projects/{}/messages:send'

logger = log.getLogger("fcm", filename="fcm.log")


def getCredentials(data):
    # Load the credentials from the dictionary
    credentials = service_account.Credentials.from_service_account_info(
        data,
        scopes=["https://www.googleapis.com/auth/firebase.messaging"])
    # Use the credentials to authenticate a Requests session
    auth_req = google.auth.transport.requests.Request()
    credentials.refresh(auth_req)
    return credentials


def sendToDevice(device, data):
    return sendData(device.cm_token, data)


def sendNotification(to_token, title, body):
    return postMessage(dict(token=to_token, notification=dict(title=title, body=body)))


def sendData(to_token, data, priority="high"):
    return postMessage(dict(token=to_token, data=data, content_available=True, priority=priority))


def postMessage(credentials, payload):
    logger.info("sending FCM", payload)
    headers = {
        'Authorization': 'Bearer ' + credentials.token,
        'Content-Type': 'application/json; UTF-8',
    }
    body = dict(message=payload)
    resp = requests.post(FCM_ENDPOINT.format(credentials.project_id), headers=headers, json=body)
    logger.info("response", resp.text)
    return resp


